var utils = require("./_utils.js");

function _nodeType(value) {
    var type = typeof (value);
    if (type == "function") {
        return type;
    } else {
        type = type == "object"
            ? (utils.isArray(value) ? "array" : "object")
            : "simple"; // assumed that all other types are simple
    }

    return type;
}

/**
 * Iterate object|array
    @param {Object} root node to iterate
    @param {String} id The id of node
    @param {Function} visitor Visitor function. Return false to stop iteration
    @param {Object} opts Iteration options
 */
function _iterateObject(root, visitor, opts) {
    switch (root.type) {
        case "object":
            var stop = visitor(root, true);
            if (!stop) {
                for (var prop in root.value) {
                    if (root.value.hasOwnProperty(prop)) {
                        if (opts.ignorePrivate && utils.startsWith.call(prop, opts.privatePreffix)) {
                            continue;
                        }

                        var value = root.value[prop];

                        var node = {
                            id: prop,
                            value: value,
                            parent: root,
                            type: _nodeType(value)
                        }
                        _iterateObject(node, visitor, opts);
                    }

                }
            }

            visitor(root, false);
            break;

        case "array":
            var stop = visitor(root, true);
            if (!stop) {
                for (var i = 0; i < root.value.length; i++) {

                    var value = root.value[i];
                    var node = {
                        id: i,
                        value: value,
                        parent: root,
                        type: _nodeType(value)
                    }
                    _iterateObject(node, visitor, opts);
                }
            }
            visitor(root, false);
            break;

        // simple types and functions not iterated
        case "simple":
            visitor(root, true);
            visitor(root, false);
            break;
        case "function":
            break;

        default:
            throw "unknown node type: " + root.type;
    }
}

function iterateObject(obj, visitor, opts) {
    opts = opts || {
        ignorePrivate: true,
        privatePreffix: "_"
    };

    var root = {
        type: _nodeType(obj),
        id: null,
        value: obj
    }

    if (root.type == "object" || root.type == "array") {
        return _iterateObject(root, visitor, opts)
    }
}

module.exports = iterateObject;