var utils = require("./_utils.js");

/**
 * Implements path splitting logic
 * @param {string} path The path
 * @returns {string[]} Array of path elements
 */
function splitPath(path, splitExp) {
    var splitExp = splitExp || /[\[\]\.\(\)]/g
    return path
        .split(splitExp)
        .filter(function (e) {
            return e.length > 0;
        })
        .map(function (prop) {
            var index = Number.parseInt(prop);
            if (Number.isNaN(index)) {
                return prop;
            }
            return index;
        });
}
module.exports.splitPath = splitPath;

/**
 * Implements query splitting logic
 * @param {string} query The query
 * @returns {string[][]} Array of [path][value] pairs
 */
function splitQuery(query, queryDelimiterChar, setValueChar) {
    var queryDelimiterChar = queryDelimiterChar || "&";
    var setValueChar = setValueChar || "=";

    var tokens = query.split(queryDelimiterChar)
        .filter(function(e){
            return e.length>0;
        })
        .map(function (token) {
            return token.split(setValueChar);
        });

    return tokens;
}
module.exports.splitQuery = splitQuery;